"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.setActionTypes = exports.DesignSystemActions = exports.DesignSystemActionType = void 0;
const tslib_1 = require("tslib");
const messages_1 = require("../messages");
const fileSystem_1 = require("./fileSystem");
const dsPrefixes = ['zero', 'rapid', 'alpha'];
var DesignSystemActionType;
(function (DesignSystemActionType) {
    DesignSystemActionType["analyzeComponentUsage"] = "Analyzed component usage";
    DesignSystemActionType["switchPrefixes"] = "Switching design system prefixes";
})(DesignSystemActionType = exports.DesignSystemActionType || (exports.DesignSystemActionType = {}));
var DesignSystemActions;
(function (DesignSystemActions) {
    /**
     * Analyses the contents of the directory and reports component usage etc.
     * @param directory - The directory to analyze.
     *
     * @public
     */
    DesignSystemActions.analyzeComponentUsage = (directory) => {
        const filePaths = fileSystem_1.FileSystemActions.listAllFiles(directory, Object.assign(Object.assign({}, fileSystem_1.FileSystemActions.defaultFileFilterOptions), { typeWhitelist: ['.ts'] }));
        const matchComponentTags = (content) => {
            const prefixes = `${dsPrefixes.join('-|')}-`;
            const searchPattern = `(?<leading>[<])(?<prefixes>${prefixes})(?<trailing>[^>|\\s|\\\\]*)`;
            const searchRegex = new RegExp(searchPattern, 'gi');
            return Array.from(content.matchAll(searchRegex), (m) => `${m.groups.prefixes}${m.groups.trailing}`);
        };
        const matchTagFor = (content) => {
            const searchPattern = `(?<tagFor>tagFor)\\((?<component>.*?)\\)`;
            const searchRegex = new RegExp(searchPattern, 'gi');
            return Array.from(content.matchAll(searchRegex), (m) => {
                const delim = '.';
                const { component } = m.groups;
                return `${component.indexOf(delim) !== -1 ? component.split(delim).pop() : component}`;
            });
        };
        let componentTags = [];
        let tagForComponents = [];
        filePaths.forEach((filePath) => {
            const content = fileSystem_1.FileSystemActions.readFile(filePath);
            componentTags = [...componentTags, ...matchComponentTags(content)];
            tagForComponents = [...tagForComponents, ...matchTagFor(content)];
        });
        (0, messages_1.componentReportUsage)(componentTags);
        (0, messages_1.designSystemComponentsRegistrationExample)(componentTags);
        (0, messages_1.tagForReportUsage)(tagForComponents, componentTags.length > 0);
    };
    /**
     * Switches all design system prefixes found in the given file to a new prefix, ie. templates, css rules, markdown etc.
     * @param filePath - The file path to operate on.
     * @param newPrefix - The new design system prefix to switch to.
     * @param oldCustomPrefix - Any custom prefix to search for in addition to the default ones.
     *
     * @public
     * @remarks
     * The search pattern avoids changing any imports or file references that may match.
     */
    DesignSystemActions.switchPrefixes = (filePath, newPrefix, oldCustomPrefix) => {
        const prefixes = `${dsPrefixes.join('-|')}-${oldCustomPrefix ? `|${oldCustomPrefix}-` : ''}`;
        const searchPattern = `(?<leading><|<\\/|\\s)(?<prefixes>${prefixes})(?<trailing>[^\\s|>]*)`;
        const searchRegex = new RegExp(searchPattern, 'gi');
        const replaceVal = `$<leading>${newPrefix}-$<trailing>`;
        fileSystem_1.FileSystemActions.replaceContent(filePath, searchRegex, replaceVal);
    };
})(DesignSystemActions = exports.DesignSystemActions || (exports.DesignSystemActions = {}));
/**
 * Add design system related actions to plop
 * @param plop - The plop api
 *
 * @public
 */
const setActionTypes = (plop) => {
    plop.setActionType(DesignSystemActionType.switchPrefixes, (answers, config) => tslib_1.__awaiter(void 0, void 0, void 0, function* () {
        const { filePath, newPrefix, oldCustomPrefix } = config;
        DesignSystemActions.switchPrefixes(filePath, newPrefix, oldCustomPrefix);
        return `to '${newPrefix}' in: ${filePath}`;
    }));
    plop.setActionType(DesignSystemActionType.analyzeComponentUsage, (answers, config) => tslib_1.__awaiter(void 0, void 0, void 0, function* () {
        DesignSystemActions.analyzeComponentUsage(config.directory);
        return '';
    }));
};
exports.setActionTypes = setActionTypes;
